@@ -5,6 +5,8 @@
 // 2011-10-30 GONG Chen <chen.sst@gmail.com>
 //
 #include <algorithm>
+#include <cfloat>
+#include <cmath>
 #include <boost/algorithm/string.hpp>
 #include <boost/lexical_cast.hpp>
 #include <boost/scope_exit.hpp>
@@ -96,23 +98,40 @@ bool UserDictEntryIterator::Release(DictEntryList* receiver) {
   return true;
 }
 
-an<DictEntry> UserDictEntryIterator::Peek() {
-  an<DictEntry> result;
-  while (!result && !exhausted()) {
-    result = (*entries_)[index_];
-    if (filter_ && !filter_(result)) {
-      ++index_;
-      result.reset();
-    }
+void UserDictEntryIterator::AddFilter(DictEntryFilter filter) {
+  DictEntryFilterBinder::AddFilter(filter);
+  // the introduced filter could invalidate the current or even all the
+  // remaining entries
+  while (!exhausted() && !filter_(Peek())) {
+    FindNextEntry();
   }
-  return result;
+}
+
+an<DictEntry> UserDictEntryIterator::Peek() {
+  if (exhausted()) {
+    return nullptr;
+  }
+  return (*entries_)[index_];
+}
+
+bool UserDictEntryIterator::FindNextEntry() {
+  if (exhausted()) {
+    return false;
+  }
+  ++index_;
+  return !exhausted();
 }
 
 bool UserDictEntryIterator::Next() {
-  if (exhausted())
+  if (!FindNextEntry()) {
     return false;
-  ++index_;
-  return exhausted();
+  }
+  while (filter_ && !filter_(Peek())) {
+    if (!FindNextEntry()) {
+      return false;
+    }
+  }
+  return true;
 }
 
 // UserDictionary members
@@ -208,7 +227,7 @@ void UserDictionary::DfsLookup(const SyllableGraph& syll_graph,
       if (i > 0 && props->type >= kAbbreviation)
         continue;
       state->credibility.push_back(
-          state->credibility.back() * props->credibility);
+          state->credibility.back() + props->credibility);
       BOOST_SCOPE_EXIT( (&state) ) {
         state->credibility.pop_back();
       }
@@ -462,10 +481,11 @@ an<DictEntry> UserDictionary::CreateDictEntry(const string& key,
   e->text = key.substr(separator_pos + 1);
   e->commit_count = v.commits;
   // TODO: argument s not defined...
-  e->weight = algo::formula_p(0,
-                              (double)v.commits / present_tick,
-                              (double)present_tick,
-                              v.dee) * credibility;
+  double weight = algo::formula_p(0,
+                                  (double)v.commits / present_tick,
+                                  (double)present_tick,
+                                  v.dee);
+  e->weight = log(weight > 0 ? weight : DBL_EPSILON) + credibility;
   if (full_code) {
     *full_code = key.substr(0, separator_pos);
   }
